<?php
/**
 * Created for Schaffen IT.
 * User: Thiago Traczykowski
 */

namespace Schaffen\ServiceOrder\ServiceOrder\Services;


class ChangeServiceOrderStatus
{
   public function alterarStatus(Request $request, $id)
   {
      try {
         DB::beginTransaction();

         $os = Os::find($id);
         if (!$os) {
            throw new SCHException("Não foi encontrada o registro código {$id}.");
         }
         $statusAtual = $os->status->sto_descricao;

         $dadosStatus = $request->input('status');
         //obtém o status para o qual está sendo alterada a OS
         $statusAlteracao = StatusOs::find($dadosStatus['status']['status']['sto_codigo']);
         if ($statusAlteracao->sto_exige_aprovacao_entrada == StatusOs::SIM) {
            $usuario = $dadosStatus['usuario'];
            //verifica o usuário e senha
            $usuarioAuth = new Usuario();
            $usuarioAuth->usu_nome = $usuario;
            $usuarioAuth->usu_senha = $dadosStatus['senha'];
            if (!$usuarioAuth->autenticaUsuario()) {
               throw new SCHException("Usuário e/ou senha inválida.");
            }
            //verifica se o usuário está entre os usuário que pode aprovar
            // a os
            $usuarioAprovacao = StatusOsUsuarioAprovacao::where("soua_usuario", $usuario)
               ->where("soua_fk_sto_codigo", $statusAlteracao->sto_codigo)->first();
            if (!$usuarioAprovacao) {
               throw new SCHException("O usuário informado não é um usuário designado para aprovação deste status.");
            }

            $totalOs = $os->total_servicos + $os->total_produtos;

            if ($usuarioAprovacao->soua_alcada < $totalOs) {
               throw new SCHException("O usuário informado não possui valor de alçada suficiente para realizar a aprovação.");
            }
         }

         $os->os_fk_sto_codigo = $statusAlteracao->sto_codigo;
         $os->save();

         // faz o controle do estoque dos itens de locação

         $this->gerenciaEstoqueLocacao($os->os_codigo);


         //verifica se o status controla estoque
         $itens = OsProdutosServicos::where("osps_fk_os_codigo", $os->os_codigo)->get();
         if ($statusAlteracao->sto_controla_estoque == StatusOs::SIM) {
            $this->gerenciaEstoqueOs($os->os_codigo, $os->os_fk_fil_codigo);
         } else {
            foreach ($itens as $item) {
               if (empty($item->osps_fk_cc_codigo_venda)) {
                  if (!empty($item->osps_fk_cc_codigo_deb)) {
                     $verificaContaCorrente = ContaCorrente::find($item->osps_fk_cc_codigo_deb);
                     if (!empty($verificaContaCorrente) && $verificaContaCorrente->cc_data_cancelamento == null) {
                        $this->cancelaMovimentacao($item->osps_fk_cc_codigo_deb, true);
                        $item->osps_fk_cc_codigo_deb = null;
                        $item->osps_fk_cc_codigo_venda = null;
                     }
                  }
                  if (!empty($item->osps_fk_cc_codigo_cred)) {
                     $verificaContaCorrente = ContaCorrente::find($item->osps_fk_cc_codigo_cred);
                     if (!empty($verificaContaCorrente) && $verificaContaCorrente->cc_data_cancelamento == null) {
                        $this->cancelaMovimentacao($item->osps_fk_cc_codigo_cred, true);
                        $item->osps_fk_cc_codigo_cred = null;
                        $item->osps_fk_cc_codigo_venda = null;
                     }
                  }
                  $item->save();
               }
            }
         }


         //verifica o texto da ocorrencia
         $textoOcorrencia = '';
         if (isset($dadosStatus['observacao']) && !empty(trim($dadosStatus['observacao']))) {
            $textoOcorrencia = "<h3>Observação:</h3>" . nl2br($dadosStatus['observacao']);
         } else {
            //verifica se o campo de observação é obrigatório
            if ($statusAlteracao->sto_exige_texto_observacao == 1) {
               throw new SCHException("O campo 'observação' deve ser informado. Informe o motivo da alteração para o status '{$statusAlteracao->sto_descricao}'.");
            }
         }

         $pessoa = Pessoa::find($os->os_fk_pes_codigo);
         $valorTotal = 0;

         if ($itens) {
            foreach ($itens as $obj) {
               $valorTotal = $valorTotal + ($obj->osps_quantidade * $obj->osps_valor_unitario);
            }
         }

         $valorTotal = number_format($valorTotal, 2);

         $statusOs = StatusOs::find($os->os_fk_sto_codigo);
         if ($statusOs && $statusOs->sto_gera_evento == 1) {
            $evento = new EventoController();
            $evento->inserirEventoOs(
               "[OS/Orçamento] - $os->os_codigo - [$pessoa->pes_nome] - agendamento", // Titulo do evento
               '<p><a onclick="document.location = \'index.php?a=ordemServico&b=visualizarOS&cod=' . $os->os_codigo . '\'">Ordem de Serviço: ' . $os->os_numero . '</a></p>' .
               '<p><a onclick="document.location = \'index.php?a=pessoa&b=visualizarPessoa&cod=' . $pessoa->pes_codigo . '\'">Cliente: ' . $pessoa->pes_nome . '</a></p>' .
               '<p>Valor total da OS: R$ ' . $valorTotal . '</p>', // Descrição do evento
               0, // Status do evento
               $os->os_data_agendamento, // Data inicial do evento
               $os->os_data_agendamento, // Data final do evento
               $os->os_fk_pes_codigo, // Dono do evento
               $os->os_codigo
            );
            $evento->inserirEventoOs(
               "[OS/Orçamento] - $os->os_codigo - [$pessoa->pes_nome] - vence hoje", // Titulo do evento
               '<p><a onclick="document.location = \'index.php?a=ordemServico&b=visualizarOS&cod=' . $os->os_codigo . '\'">Ordem de Serviço: ' . $os->os_numero . '</a></p>' .
               '<p><a onclick="document.location = \'index.php?a=pessoa&b=visualizarPessoa&cod=' . $pessoa->pes_codigo . '\'">Cliente: ' . $pessoa->pes_nome . '</a></p>' .
               '<p>Valor total da OS: R$ ' . $valorTotal . '</p>', // Descrição do evento
               0, // Status do evento
               $os->os_data_validade, // Data inicial do evento
               $os->os_data_validade, // Data final do evento
               $os->os_fk_pes_codigo, // Dono do evento
               $os->os_codigo
            );
         }

         //registra o log
         $tipoOcorrencia = new OsLog([
            "osl_fk_os_codigo" => $os->os_codigo,
            "osl_fk_too_codigo" => TipoOcorrenciaOs::STATUS,
            "osl_ocorrencia" =>
               "Status alterado de {$statusAtual} para {$statusAlteracao->sto_descricao}." . $textoOcorrencia,
            "osl_codigo_operacao" => $os->os_codigo
         ]);
         $tipoOcorrencia->save();

         $kanbar = new Kanban();
         //Dispara evento de alteração da OS
         $kanbar->alteracaoOs($os->os_codigo);
         DB::commit();


         // Verifica se o usuário é admin do módulo de os e se o status permite faturamento
         $usuario = Usuario::find(UsuarioAuth::getCodigoUsuario());
         $permiteDesconto = $usuario->usu_fk_gru_codigo == 1 || $usuario->usu_adm_ordem_servico == 1 || $statusAlteracao->sto_permite_desconto == 1;

         return Response::success("Status alterado com sucesso.", ['permiteDesconto' => $permiteDesconto]);
      } catch (SCHException $e) {
         DB::rollback();
         return Response::faillure($e->getMessage(), $e->getTrace());
      } catch (\Exception $e) {
         DB::rollback();
         return Response::error($e->getMessage(), $e);
      }
   }
}
