<?php
/**
 * Created for Schaffen IT.
 * User: Thiago Traczykowski
 */

namespace Schaffen\ServiceOrder\ServiceOrder\Services;


use Illuminate\Support\Str;
use Schaffen\Core\Person\Repositories\PersonRepository;
use Schaffen\ServiceOrder\ServiceOrder\Repositories\ServiceOrderRepository;
use Schaffen\ServiceOrder\ServiceOrder\Repositories\ServiceOrderResourceRepository;
use Schaffen\ServiceOrder\ServiceOrder\Traits\SetServiceOrderOnAssociate;

class CreateOrUpdateServiceOrderResource
{

    use SetServiceOrderOnAssociate;

    /**
     * @var ServiceOrderResourceRepository
     */
    private ServiceOrderResourceRepository $service_order_resource_repository;
    /**
     * @var PersonRepository
     */
    private PersonRepository $person_repository;
    /**
     * @var ServiceOrderRepository
     */
    private ServiceOrderRepository $service_order_repository;

    public function __construct(
        ServiceOrderResourceRepository $service_order_resource_repository,
        PersonRepository $person_repository
    )
    {

        $this->service_order_resource_repository = $service_order_resource_repository;
        $this->person_repository = $person_repository;
    }

    public function handleResources(
        $resources, $service_order_id
    )
    {
        $resources = !is_array($resources) ? [$resources] : $resources;
        $return = [];

        foreach ($resources as $resource) {
            $os_filter = ['osr_fk_os_codigo' => $service_order_id];

            $resource_finded = $this->service_order_resource_repository->first(array_merge($resource, $os_filter));

            if (is_null($resource_finded)) {
                $return[] = $this->service_order_resource_repository->storeOrUpdate(
                    $resource, $resource['osr_codigo'] ?? NULL,
                    $this->getServiceOrderRelation($service_order_id)
                );
            } else {
                $return[] = $resource_finded;
            }

        }
        return $return;
    }
}
