<?php
/**
 * Created for Schaffen IT.
 * User: Thiago Traczykowski
 */

namespace Schaffen\ServiceOrder\ServiceOrder\Services;


class GenerateSaleFromServiceOrder
{


   /**
    * A partir dos dados informados na tela de faturamento (itens, forma de pagamento)
    * gera uma venda e grava um faturamento fazendo os devidos vinculos com a
    * OS que originou a operação.
    *
    * @param Request $request
    * @param type $id
    * @return type
    * @throws SCHException
    */
   public function gerarVendaItensOs(Request $request, $id)
   {
      try {
         DB::beginTransaction();
         $os = Os::find($id);
         if (!$os) {
            throw new SCHException("A ordem de serviço código {$id} não foi localizada.");
         }

         $cfops = $request->input("cfops");

         //obtém o local de estoque padrão da OS
         $localEstoqueOS = LocalEstoque::where("loe_fk_fil_codigo", $os->os_fk_fil_codigo)
            ->where("loe_tipo", LocalEstoque::LOCAL_ORDEM_SERVICO)->first();

         $codigoLocalEstoque = null;
         if ($localEstoqueOS) {
            $codigoLocalEstoque = $localEstoqueOS->loe_codigo;
         }

         if (!isset($os->os_recursos) || empty($os->os_recursos) || count($os->os_recursos) == 0) {
            $usuario = Usuario::find($os->os_fk_usu_codigo_vendedor);
            $codigoPessoaVendedor = $usuario->usu_fk_pes_codigo;
         } else {
            $codigoPessoaVendedor = $os->os_recursos[0]['pes_codigo'];
         }


         // monta o texto de observação da venda
         $textoObservacao = "";
         if ($os->os_tipo == Os::TIPO_ORCAMENTO) {
            $textoObservacao .= "Nº Orc {$os->os_numero_tipo}<br/>";
            if (isset($os->os_campos) || !empty($os->os_campos)) {
               foreach ($os->os_campos as $campo) {
                  if ($campo["campo"]["caos_descricao"] == "Observação") {
                     $textoObservacao .= " - Obs Orc: " . strip_tags($campo["valor"]) . "<br/>";
                  } else if ($campo["campo"]["caos_descricao"] == "Data de entrega") {
                     $textoObservacao .= " - Data entrega Orc : " . utf8_decode(strip_tags($campo["valor"])) . " ";
                  }
               }
            }


         } else {
            $textoObservacao .= "OS {$os->os_numero_tipo}<br/>";
            if (isset($os->os_campos) || !empty($os->os_campos)) {
               foreach ($os->os_campos as $campo) {
                  if ($campo["campo"]["caos_descricao"] == "Observação") {
                     $textoObservacao .= strip_tags($campo["valor"]) . "<br/>";
                  } else if ($campo["campo"]["caos_descricao"] == "Data de entrega") {
                     $textoObservacao .= "<b>&nbsp;&nbsp;Data entrega OS: " . utf8_decode(strip_tags($campo["valor"])) . "</b> ";
                  }
               }
            }
         }

         // monta o objeto de venda para realizar a venda
         $dadosVenda = [
            "ven_fk_pes_codigo" => $os->os_fk_pes_codigo,
            "ven_fk_loe_codigo" => $codigoLocalEstoque,
            "ven_fk_ope_codigo" => $request->input("natureza_operacao"),
            "ven_fk_fil_codigo" => $os->os_fk_fil_codigo,
            "ven_fk_pes_codigo_vendedor" => $codigoPessoaVendedor,
            "ven_observacao" => $textoObservacao,
            "ven_tipo_venda" => $request->input("tipo_venda"),
            "ven_cpf_cnpj" => $os->cliente->pes_identificador,
            "ven_modulo" => Venda::MODULO_OS,
            "itens" => [],
            "formas_pagamento" => $request->input("formas_pagamento"),
            "amb_codigo" => $request->input("amb_codigo"),
            "venda_balcao" => $request->input("venda_balcao") //Definir a venda balcão como 1 para que seja possível gerar informações na conta corrente do módulo caixa (quando habilitado)
         ];
         // pega os itens selecionados para faturamento e monta os itens da venda
         $totalVenda = 0;
         foreach ($request->input("itens") as $codigoItemOs) {
            $item = OsProdutosServicos::find($codigoItemOs);
            if (!$item) {
               throw new SCHException("Item código {$codigoItemOs} não encontrado na OS/Orçamento {$id}");
            }
            if (!empty($item->osps_fk_fat_codigo)) {
               $mensagem = "O item {$item->produto->nome_completo}";
               $mensagem .= " (quantidade: " . SCHTratamentoTipos::exibirNumber($item->osps_quantidade) . ") ";
               $mensagem .= " já encontra-se faturado.";
               throw new SCHException($mensagem);
            }
            $dadosVenda["itens"][] = [
               "vei_fk_pve_codigo" => $item->osps_fk_pve_codigo,
               "vei_fk_cfop_codigo" => $cfops[$item->osps_codigo],
               "vei_quantidade" => $item->osps_quantidade,
               "vei_valor_unitario" => $item->osps_valor_unitario,
               "vei_desconto" => $item->osps_desconto,
               "vei_serial" => $item->osps_serial
            ];
            $totalVenda += ($item->osps_quantidade * $item->osps_valor_unitario) - $item->osps_desconto;
         }

         // gera a venda
         $vendaController = new \App\Http\Controllers\Api\Estoque\Movimentacao\VendaController();
         $venda = $vendaController->realizaVenda($dadosVenda);

         // insere um faturamento
         // /* Cria um faturamento para relacionar as mvfs das parcelas com o faturamento */
         $faturamento = new Faturamento();
         $faturamento->fat_fk_pes_codigo = $os->os_fk_pes_codigo;
         $faturamento->fat_fk_fil_codigo = $os->os_fk_fil_codigo;
         $faturamento->save();
         // vincula faturamento com a OS
         $faturamentoOs = new FaturamentoOs();
         $faturamentoOs->fao_fk_os_codigo = $os->os_codigo;
         $faturamentoOs->fao_fk_fat_codigo = $faturamento->fat_codigo;
         $faturamentoOs->save();
         // vincula a venda com o faturamento
         $faturamentoVenda = new FaturamentoVenda();
         $faturamentoVenda->fav_fk_fat_codigo = $faturamento->fat_codigo;
         $faturamentoVenda->fav_fk_ven_codigo = $venda->ven_codigo;
         $faturamentoVenda->save();
         // vincula os itens faturados com o faturamento e com a venda
         foreach ($request->input("itens") as $codigoItemOs) {
            $itemOs = OsProdutosServicos::find($codigoItemOs);
            if (Parametro::getDadosParametro(508)->par_valor == '1') {
               $snapshot = Snapshot::where('sna_fk_pve_codigo', $itemOs->osps_fk_pve_codigo)->first();
               if (empty($snapshot)) {
                  throw new SCHException("Não existe nenhuma movimentação de entrada/ajuste estoque deste produto.");
               }

            }
            $itemOs->osps_fk_ven_codigo = $venda->ven_codigo;
            $itemOs->osps_fk_cc_codigo_venda = $venda->ven_codigo;
            $itemOs->osps_fk_fat_codigo = $faturamento->fat_codigo;
            $itemOs->osps_fk_cc_codigo_cred = null;
            $itemOs->osps_fk_cc_codigo_deb = null;
            foreach ($venda->item->item as $v) {
               if ($itemOs->osps_fk_pve_codigo == $v->vei_fk_pve_codigo) {
                  $itemOs->osps_fk_cc_codigo_deb = $v->vei_fk_cc_codigo;
                  $itemOs->osps_fk_cfop_codigo = $v->vei_fk_cfop_codigo;
               }
            }
            $itemOs->save();
         }
         // grava o log com as informações da venda
         $logOs = new OsLog([
            "osl_fk_os_codigo" => $os->os_codigo,
            "osl_fk_too_codigo" => TipoOcorrenciaOs::VENDA,
            "osl_ocorrencia" => "Venda código #{$venda->ven_codigo} gerada via "
               . "faturamento código #{$faturamento->fat_codigo} "
               . "no valor de " . SCHTratamentoTipos::exibirNumber($totalVenda),
            "osl_codigo_operacao" => $venda->ven_codigo
         ]);
         $logOs->save();
         DB::commit();
         $mensagemRetorno = "Venda gerada com sucesso.";
      } catch (SCHException $e) {
         DB::rollback();
         return Response::faillure($e->getMessage(), $e->getTrace());
      } catch (\Exception $e) {
         DB::rollback();
         return Response::error($e->getMessage(), $e);
      }

      // verifica se é uma NFCe - se sim, faz a chamada a função que gera a
      $codigoNotaFiscal = null;
      $falhaGerarNF = false;
      if ($venda->ven_tipo_venda == Venda::TIPO_NFCE) {
         $vendaCtrl = new \App\Http\Controllers\Api\Estoque\Movimentacao\VendaController();
         $falhaGerarNF = true;
         try {
            $codigoNotaFiscal = $vendaCtrl->gerarNotaFiscalVenda($venda->ven_codigo, true);
            $documento = Documento::where("doc_fk_nof_codigo", $codigoNotaFiscal)->first();
            //verifica o status da NF
            if (!$documento) {
               throw new SCHException("Acesse o relatório de vendas para gerar a nota fiscal desta venda.");
            }

            if ($documento->doc_status == 100) {
               $falhaGerarNF = false;
            } else {
               $mensagemRetorno .= "Venda gerada com sucesso, porém ao executar o processo de NFCe ocorreu um erro: acesse o relatório de vendas para gerar a nota fiscal desta venda.";
            }
         } catch (\Exception $e) {
            $mensagemRetorno = "Venda gerada com sucesso, porém ao executar o processo de NFCe ocorreu um erro: " . $e->getMessage();
         }
      }

      $this->verificaFaturamento($id);

      return Response::success($mensagemRetorno, [
            "ven_codigo" => $venda->ven_codigo,
            "nof_codigo" => $codigoNotaFiscal,
            "fat_codigo" => $faturamento->fat_codigo,
            "falha_nf" => $falhaGerarNF
         ]
      );
   }

}
